// Change the speed of rotation and the size of the orb by moving the mouse.
// Its position also controls the sound generated by GameSynth in real-time via TCP. 

import processing.net.*; 

// graphic settings
int count = 20;
float[] pt;
int[] colors;

// TCP settings
Client gsClient; 
String IP = "127.0.0.1";
int port = 28542;
char delimiter = char(13); 

// real-time settings
float x;
float y;
int playing = 0;
float radius = 0.0f;
float speed = 1.0f;

void setup()
{  
    // connect to the GameSynth tool (needs to be already running) 
    gsClient = new Client(this,IP,port);
    if(!gsClient.active())
    {
        String message = "Could not connect to GameSynth. Please make sure it is running and the port is set to 28452."; 
        javax.swing.JOptionPane.showMessageDialog(null, message);
        exit();
    }
    
    // load a patch from the sketch folder
    gsClient.write("load_patch \""+sketchPath("")+"Gravity Orb.gspatch\"" + delimiter);  
    while(gsClient.available()==0) {}
    String data = gsClient.readString();
    if(!data.substring(0,2).equals("OK")) 
    {
        String message = "Could not find the GameSynth patch requested."; 
        javax.swing.JOptionPane.showMessageDialog(null, message);
        exit();  
    }

    // prepare the screen
    size(1024, 768, P3D);
    background(255);
    smooth(8);

    // prepare the arc parameters
    pt = new float[5 * count];
    colors = new int[count];  
    randomSeed(30);
    int index = 0;
    for (int i = 0; i < count; i++) 
    {
        pt[index++] = random(TAU);               // X axis rotation
        pt[index++] = random(TAU);               // Y axis rotation
        pt[index++] = int(random(2,50)*5);       // radius
        pt[index++] = random(4,32);              // width of the arc
        pt[index++] = radians(random(5,30)) / 5; // speed of rotation      
        colors[i] = color (220,170,250,random(80,200));
    }
}

// if a key is pressed we quit the program
void keyPressed() 
{
    // exit if a key is pressed (and terminate the connection)
    gsClient.write("stop" + delimiter);
    gsClient.stop();
    gsClient.clear();
    exit();
}

// when the mouse is pressed, we send values to the meta-parameters to change the sound based on the mouse position 
void mouseDragged()
{
    // if not playing the patch yet, start it 
    if(playing == 0) 
    {
        gsClient.write("play" + delimiter);
        playing = 1;
    }
  
    // caculate the coordinates in the range [0...1] 
    x = (float)mouseX / width;
    y = 1.0f - (float)mouseY / height;
  
    // send them to the 3 meta-parameters of the patch  
    String message = "set_metavalue BY_INDEX 0 " + str(x) + delimiter;
    gsClient.write(message);
    
    message = "set_metavalue BY_INDEX 1 " + str(y) + delimiter;
    gsClient.write(message);

    message = "set_metavalue BY_INDEX 2 " + str(x) + delimiter;
    gsClient.write(message);
}

// stop the sound when the mouse button is released 
void mouseReleased() 
{
    gsClient.write("stop" + delimiter);
    playing = 0;
}

// draw a frame 
void draw() 
{   
    background(0);
  
    translate(width/2, height/2, 0);
    rotateX(PI / 6);
    rotateY(PI / 6);
  
    // draw the entral sphere
    lights();
    fill(color(200,150,230,100+y*50));
    sphere(112);
  
    // draw the arcs around the sphere
    int index = 0;
    for (int i = 0; i < count; i++) 
    {
        pushMatrix();
        rotateX(pt[index++]);
        rotateY(pt[index++]);
     //<>//
        CalculateSpeedAndRadius();
            
        fill(colors[i]);
        noStroke();
        drawArc(0, 0, pt[index++] * radius, pt[index++]);
        
        // increase rotation
        pt[index-4] += pt[index] / speed;
        pt[index-3] += pt[index++] / speed;
    
        popMatrix();
    }
}

// calculate the new speed and radius
void CalculateSpeedAndRadius()
{
    
    // when playing a patch, they are based on the mouse coordinates
    if(playing == 1)
    {
        radius = 0.5f * (1.0f + x);
        speed = 0.1f + 2 * (1.0f - y);
    }
    // otherwise we bring them back slowly to their default values 
    else
    {     
        radius -= 0.001f;      
        if(radius < 0.5f)
          radius = 0.5f;
          
        speed += 0.001f;
        if(speed > 1)
          speed = 1f;
   }
}

// draw an arc around the sphere
void drawArc(float x, float y, float radius, float arcWidth) 
{
    beginShape(QUAD_STRIP);
    for (int i = 0; i <= 360; i++) 
    {
        float angle = radians(i);
        vertex(x + cos(angle) * radius, y + sin(angle) * radius);
        vertex(x + cos(angle) * (radius + arcWidth), y + sin(angle) * (radius + arcWidth));
    }
    endShape();
}
